@@ -41,7 +41,6 @@ Ext.namespace("Ext.ux");
 
 Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
     toolsPanelWidth: 100,
-    enableScrolling : false,
     initComponent: function(config) {
         Ext.ux.StartMenu.superclass.initComponent.call(this, config);
 		this.items = new Ext.util.MixedCollection();
@@ -50,7 +49,6 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         if(tools){
             this.addTool.apply(this, tools);
         }
-        this.useMenuSearch = this.initialConfig.useMenuSearch;
     },
 
     // private
@@ -89,54 +87,14 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         var mc = this.menuBWrap.wrap({
             cls: "x-window-mc ux-start-menu-bwrap"
         });
-        this.mainMenuPanel = this.menuBWrap.createChild({
-            tag: "div",
-            cls: "x-panel",
-            style: {float: 'left'},
-        });
-        this.menuPanel = this.mainMenuPanel.createChild({
-            tag: "div",
-            cls: "x-panel ux-start-menu-apps-panel"
-        });
-
-        // Формирование поля для поиска и панели для него
-        if (this.useMenuSearch) {
-            this.menuSearchField = new Ext.app.form.SearchField({
-                emptyText: 'Поиск',
-                hideLabel: true,
-                triggerClass: 'x-form-search-trigger',
-                startMenu: this,
-                enableKeyEvents: true,
-                onTrigger1Click: function () {
-                    // Очистка поля для поиска и возвращение исходного меню
-                    this.setValue('');
-                    this.triggers[0].hide();
-                    this.startMenu.closeAllFloatMenu();
-                    this.startMenu.clearMenuSearchFilter();
-                },
-                onTrigger2Click: function () {
-                    // Установка фильтрации элементов меню по тексту в поле
-                    var filterText = this.getValue(),
-                        trigger = this.triggers[0];
-                    filterText ? trigger.show() : trigger.hide();
-                    this.startMenu.showFilteredStartMenuItems(filterText);
-                },
-            });
-            this.menuSearchField.on('keydown', function () {
-                this.startMenu.closeAllFloatMenu();
-            });
-            this.menuSearchPanel = new Ext.FormPanel({
-                cls: 'x-menu-list-item',
-                items: [this.menuSearchField],
-                style: {'background-color': 'white'},
-            });
-            this.menuSearchPanel.render(this.mainMenuPanel);
-        }
 
+        this.menuPanel = this.menuBWrap.createChild({
+            tag: "div",
+            cls: "x-panel x-border-panel ux-start-menu-apps-panel"
+        });
         this.toolsPanel = this.menuBWrap.createChild({
             tag: "div",
-            cls: "x-panel ux-start-menu-tools-panel",
-            style: {float: 'left', height: '100%'},
+            cls: "x-panel x-border-panel ux-start-menu-tools-panel"
         });
 
         var bwrap = ml.wrap({cls: "x-window-bwrap"});
@@ -182,11 +140,13 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
 
         this.menuPanel.setStyle({
             padding: '2px',
+            position: 'absolute',
             overflow: 'auto'
         });
 
         this.toolsPanel.setStyle({
             padding: '2px 4px 2px 2px',
+            position: 'absolute',
             overflow: 'auto'
         });
 
@@ -216,34 +176,23 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         this.parentMenu = parentMenu;
         if(!this.el){
             this.render();
-        } else {
-            if (this.useMenuSearch) {
-                this.menuSearchField.clear();
-            }
         }
 
         this.fireEvent("beforeshow", this);
-        var posArray = this.el.getAlignToXY(el, pos || this.defaultAlign);
+        var posArray = this.el.getAlignToXY(el, pos || this.defaultAlign)
         //kir add 15.03.2011
         // Если taskbar находится вверху, делается переназначение положения.
-        posArray[1] = posArray[1] < 0 ? el.getHeight() : posArray[1];
+        posArray[1] = posArray[1] < 0 ? el.getHeight() : posArray[1]
         this.showAt(posArray ,parentMenu, false);
 
-        var box = this.menuBWrap.getBox(),
-            leftMenuWidth = box.width - this.toolsPanelWidth,
-            menuPanelHeight = box.height;
+        var tPanelWidth = this.toolsPanelWidth;
+        var box = this.menuBWrap.getBox();
+        this.menuPanel.setWidth(box.width-tPanelWidth);
+        this.menuPanel.setHeight(box.height);
 
-        if (this.useMenuSearch) {
-            this.menuSearchPanel.setWidth(leftMenuWidth);
-            this.menuSearchField.itemCt.setStyle({margin: '2px'});
-            menuPanelHeight -= this.menuSearchPanel.getHeight();
-            // Нужно для установки всей ширины
-            this.menuSearchField.wrap.setWidth('100%');
-            this.menuSearchField.triggers[1].show();
-        }
-        this.menuPanel.setHeight(menuPanelHeight);
-        this.menuPanel.setWidth(leftMenuWidth);
-        this.toolsPanel.setWidth(this.toolsPanelWidth);
+        this.toolsPanel.setWidth(tPanelWidth);
+        this.toolsPanel.setX(box.x+box.width-tPanelWidth);
+        this.toolsPanel.setHeight(box.height);
     },
 
     addTool : function(){
@@ -304,109 +253,7 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
         this.title = title;
         this.header.child('span').update(title);
         return this;
-    },
-
-    /**
-     * Закрывает все выпадающие меню, чтобы после фильтрации не возникали баги,
-     * связанные с его расположением
-     * @param complexMenuItem Элемент меню с вложенными элементами.
-     * Если параметр не указан, выбирается основное меню.
-     */
-    closeAllFloatMenu: function (complexMenuItem) {
-        var menuItem = complexMenuItem || this;
-        menuItem.items.each(function (item) {
-            var menu = item.menu;
-
-            if (menu) {
-                if (menu.ul && menu.ul.isVisible()) {
-                    item.deactivate();
-                }
-                this.closeAllFloatMenu(menu);
-            }
-        }.bind(this));
-    },
-
-    /**
-     * Устанавливает/отключает видимость элемента меню в соответствии
-     * с текстом для фильтрации
-     * @param filterText Текст для фильтрации
-     * @param menuItem Пункт меню для проверки. Если он содержит дочерние
-     *  элементы, они также проверяются.
-     * @returns {boolean} Стал ли элемент видимым или нет
-     */
-    applyMenuFilter: function (filterText, menuItem) {
-        var text = filterText.toLowerCase(),
-            isVisible = menuItem.text.toLowerCase().indexOf(text) !== -1,
-            currentMenu = menuItem.menu;
-
-        if (currentMenu) {
-            var innerVisibleCount = this.showFilteredStartMenuItems(
-                filterText, currentMenu)
-
-            // Если у вложенного меню внутри есть хотя бы 1
-            // видимый элемент, то это меню тоже видно
-            if (!isVisible && innerVisibleCount > 0) {
-                isVisible = true;
-            }
-
-            if (currentMenu.ul) {
-                currentMenu.ul.setHeight('auto');
-            }
-        }
-        menuItem.setVisible(isVisible);
-        return isVisible;
-    },
-
-    /**
-     * Отфильтровывает пункты меню в соответствии со строкой
-     * @param filterText Текст для фильтрации. Если пустой, фильтр отключается
-     * @param complexMenuItem Пункт меню со вложенными элементами для
-     *  проверки (необязательный параметр). Если не задан, проверяются корневые
-     *  и вложенные в них элементы меню.
-     * @returns {number} Количество видимых элементов внутри данного пункта меню
-     */
-    showFilteredStartMenuItems: function (filterText, complexMenuItem) {
-        var menuItem = complexMenuItem || this,
-            innerVisibleCount = 0;
-
-        if (!filterText) {
-            this.clearMenuSearchFilter();
-            return menuItem.items.length;
-        }
-
-        menuItem.items.each(function (item){
-            innerVisibleCount += Number(this.applyMenuFilter(filterText, item));
-        }.bind(this));
-
-        return innerVisibleCount
-    },
-
-    /**
-     * Делает пункт меню видимым, то есть убирает эффект фильтрации
-     * @param menuItem Пункт меню. Если он содержит дочерние
-     *  элементы, они также обрабатываются.
-     */
-    clearMenuItemSearchFilter: function (menuItem) {
-        if (menuItem.menu) {
-            this.clearMenuSearchFilter(menuItem.menu);
-
-            if (menuItem.menu.ul) {
-                menuItem.menu.ul.setHeight('auto');
-            }
-        }
-        menuItem.setVisible(true);
-    },
-
-    /**
-     * Делает пункты меню видимыми, то есть убирает эффект фильтрации
-     * @param complexMenuItem Пункт меню со вложенными элементами, для которого
-     *  отключается фильтрация. Если не указан параметр, то обрабатываются
-     *  все элементы меню.
-     */
-    clearMenuSearchFilter: function (complexMenuItem) {
-        var menuItem = complexMenuItem || this;
-        menuItem.items.each(this.clearMenuItemSearchFilter.bind(this));
-    },
+    }
 });
 
 
@@ -423,7 +270,7 @@ Ext.ux.StartMenu = Ext.extend(Ext.menu.Menu, {
 Ext.ux.TaskBar = function(app){
     this.app = app;
     this.init();
-};
+}
 
 Ext.extend(Ext.ux.TaskBar, Ext.util.Observable, {
     init : function(){
@@ -432,8 +279,7 @@ Ext.extend(Ext.ux.TaskBar, Ext.util.Observable, {
             height: 360,
             shadow: true,
             title: 'Jack Slocum',
-            width: 300,
-            useMenuSearch: this.app.useMenuSearch,
+            width: 300
         }, this.app.startConfig));
 
         this.startBtn = new Ext.Button({
@@ -548,6 +394,7 @@ Ext.ux.TaskButtonsPanel = Ext.extend(Ext.BoxComponent, {
     initComponent : function() {
         Ext.ux.TaskButtonsPanel.superclass.initComponent.call(this);
         this.on('resize', this.delegateUpdates);
+
         this.items = [];
 
         this.stripWrap = Ext.get(this.el).createChild({
@@ -877,16 +724,16 @@ Ext.extend(Ext.ux.TaskBar.TaskButton, Ext.Button, {
         this.cmenu.on('beforeshow', function(){
             var items = this.cmenu.items.items;
             var w = this.win;
-            if (w.masked) {
-                for (var i = 0, len = items.length; i < len; i++) {
-                    items[i].setDisabled(true);
-                }
-            }
-            else {
-                items[0].setDisabled(w.maximized !== true && w.hidden !== true);
-                items[1].setDisabled(w.minimized === true || !w.minimizable);
-                items[2].setDisabled(w.maximized === true || w.hidden === true || !w.maximizable);
-            }
+			if (w.masked) {
+				 for (var i = 0, len = items.length; i < len; i++) {
+				 	items[i].setDisabled(true);
+				 }
+			}
+			else {
+				items[0].setDisabled(w.maximized !== true && w.hidden !== true);
+				items[1].setDisabled(w.minimized === true || !w.minimizable);
+				items[2].setDisabled(w.maximized === true || w.hidden === true || !w.maximizable);
+			}
         }, this);
 
         this.el.on('contextmenu', function(e){
@@ -899,9 +746,6 @@ Ext.extend(Ext.ux.TaskBar.TaskButton, Ext.Button, {
             xy[1] -= this.cmenu.el.getHeight();
             this.cmenu.showAt(xy);
         }, this);
-        this.win.on('close', function(){
-            this.cmenu.destroy();
-        }, this);
     },
 
     closeWin : function(cMenu, e, win){
@@ -916,13 +760,13 @@ Ext.extend(Ext.ux.TaskBar.TaskButton, Ext.Button, {
 
 
 Ext.ux.Clock = Ext.extend(Ext.Toolbar.TextItem,{
-    shortdays: ["Вс", "Пн", "Вт", "Ср", "Чт", "Пт", "Сб"],
+	shortdays: ["Вс", "Пн", "Вт", "Ср", "Чт", "Пт", "Сб"],
     currTime: function(){
-        var d = new Date();
-        var dateStr = d.format('d (XXX) M Y G:i:s');
-        var day = this.shortdays[d.getDay()];
-        return dateStr.replace('XXX', day);
-    }
+		var d = new Date();
+		var dateStr = d.format('d (XXX) M Y G:i:s');
+		var day = this.shortdays[d.getDay()];
+		return dateStr.replace('XXX', day);
+	}
     ,initComponent: function() {
         Ext.apply(this, {
             text: this.currTime()
@@ -935,7 +779,7 @@ Ext.ux.Clock = Ext.extend(Ext.Toolbar.TextItem,{
             run: this.update,
             scope: this,
             interval: 1000 //1 second
-        };
+        }
         Ext.TaskMgr.start(this.clock_updater);
     }
     ,update: function() {
@@ -944,148 +788,25 @@ Ext.ux.Clock = Ext.extend(Ext.Toolbar.TextItem,{
 });
 Ext.reg('ux_clock', Ext.ux.Clock);
 
-function getDesktopComponent(el) {
-    var module = AppDesktop.getModule(el.id.replace('-shortcut', ''));
-    if(module){
-        return module.launcher;
-    } else {
-        // если не нашли в модулях, то поищем вообще в компонентах
-        return Ext.getCmp(el.id.replace(/-shortcut\d*/, ''));
-    }
-}
-
 /*!
  * Ext JS Library 3.3.0
  * Copyright(c) 2006-2010 Ext JS, Inc.
  * licensing@extjs.com
  * http://www.extjs.com/license
  */
+Ext.Desktop = function(app){
+    var taskbar,
+        desktopEl = Ext.get('x-desktop'),
+        taskbarEl = Ext.get('ux-taskbar'),
+        shortcuts = Ext.get('x-shortcuts');
 
-TopToolbar = Ext.extend(Ext.Panel, {
-    monitorResize: true,
-    autoWidth: true,
-    autoHeight: true,
-    height: 0,
-    style: 'margin-top:0px',
-    bodyStyle: 'padding:0px',
-    renderTo: 'ux-toptoolbar',
-    autoScroll: true,
-    tbar: [],
+    this.taskbar = taskbar = new Ext.ux.TaskBar(app);
+    this.xTickSize = this.yTickSize = 1;
 
-    initComponent : function() {
-        TopToolbar.superclass.initComponent.call(this);
-        Ext.EventManager.onWindowResize(this.fireResize, this);
-    },
-    fireResize : function(w, h){
-        this.onResize(w, 0, w, 0);
-        this.fireEvent('resize', this, w, h, w, h);
-    },
-    beforeDestroy : function(){
-        TopToolbar.superclass.beforeDestroy.call(this);
-        Ext.EventManager.removeResizeListener(this.fireResize, this);
-    }
-});
-
-Ext.define('Ext.Desktop', {
-    constructor: function (app) {
-        this.primaryEvent = app.primaryEvent || 'click';
-        this.enableDragAndDrop = app.enableDragAndDrop;
-        this.activeWindow = undefined;
-        this.windowManager = new Ext.WindowGroup();
-        this.taskbar = new Ext.ux.TaskBar(app);
-        this.xTickSize = this.yTickSize = 1;
-
-        var shortcuts = Ext.get('x-shortcuts');
-        if(shortcuts){
-            app.on('ready', function(){
-                Ext.each(shortcuts.query('td'), function(item){
-                    this.prepareDesktopItem(Ext.get(item));
-                }, this);
-            }, this);
-        }
-
-        //ZIgi 16.12 дабы окна рендерились только внутри десктопа
-        //оставляя верхний и нижний тулбары
-        Ext.override(Ext.Window,
-        {
-            renderTo: 'x-desktop'
-        });
-
-        // В ИЕ7 не поддерживается display: inline-block
-        // и из-за этого ярлыки на рабочем столе выстраиваются в одну линию
-        // Этот недостаток предотвращается навешиванием на ресайз обработчика:
-        if (Ext.isIE7) {
-            this.tuneForIE7();
-        }
-    },
-
-    addModules: function (modules) {
-        var toolsNotCreated = true;
-        if (modules.length > 0) {
-        var tbar;
-            for (var i = 0; i < modules.length; i++) {
-                var m = modules[i];
-                if (m && m.id) {
-                    if (m.id.indexOf('toptoolbar-item') === 0) {
-                        if(toolsNotCreated){
-                            this.topToolBarPanel = new TopToolbar();
-                            tbar = this.topToolBarPanel.getTopToolbar();
-                            // добавим функционал друг энд глюк
-                                tbar.on('add', function (toolbar, cmp) {
-                                // компонент можно тянуть, если у него есть признак :)
-                                if (cmp.dragable === true){
-                                    cmp.on('render', function(){
-                                        var dd = new Ext.dd.DragSource(this.el,{
-                                            ddGroup: 'toolbaritems',
-                                            scroll: false,
-                                            dragData: cmp,
-                                            onStartDrag: function(){
-                                                // закроем все меню при начале перетаскивания
-                                                tbar.cascade(function () {
-                                                    if (this.hideMenu) this.hideMenu();
-                                                });
-                                            }
-                                        });
-                                    }, cmp);
-                                }
-                            });
-                            toolsNotCreated = false;
-                        }
-                        if(m.launcher.text == 'FILLBLOCK'){
-                            tbar.add('->');
-                        } else if(m.launcher.text == 'TIMEBLOCK'){
-                            var clock = new Ext.ux.Clock();
-                            tbar.add(clock);
-                        } else if(m.launcher.text == '-'){
-                            tbar.add('-');
-                        } else if(m.launcher.text == 'UI_OBJECT'){
-                            tbar.add(m.launcher.ui_object);
-                        } else {
-                            tbar.add({
-                                scale: 'small'
-                               ,iconAlign: 'left'
-                               ,text: m.launcher.text
-                               ,iconCls: m.launcher.iconCls
-                               ,handler: m.launcher.handler
-                               ,menu: m.launcher.menu
-                               ,tooltip: m.launcher.tooltip
-                               ,dragable: this.enableDragAndDrop
-                               ,dragData: m
-                            });
-                        }
-                    }
-                }
-            }
-            if(!toolsNotCreated){
-                this.topToolBarPanel.doLayout();
-            }
-        }
-
-        this.layout();
-        Ext.EventManager.onWindowResize(this.layout);
-    },
-
-    tuneForIE7: function () {
+    // В ИЕ7 не поддерживается display: inline-block
+    // и из-за этого ярлыки на рабочем столе выстраиваются в одну линию
+    // Этот недостаток предотвращается навешиванием на ресайз обработчика:
+    if (Ext.isIE7) {
         Ext.EventManager.onWindowResize(function () {
             var box,
                 shortcutsElements,
@@ -1133,123 +854,170 @@ Ext.define('Ext.Desktop', {
                 }
             }
         });
-        Ext.EventManager.fireResize();
-    },
+    }
 
-    minimizeWin: function (win) {
+
+    //ZIgi 16.12 дабы окна рендерились только внутри десктопа
+    //оставляя верхний и нижний тулбары
+    Ext.override(Ext.Window,
+    {
+        renderTo: 'x-desktop'
+    });
+
+    var windows = new Ext.WindowGroup();
+    var activeWindow;
+
+    var toptoolbarEl = Ext.get('ux-toptoolbar');
+    var ms = app.getModules();
+    var toptools = [];
+    Ext.each(ms, function(m){
+        if (m && m.id && m.id.indexOf('toptoolbar-item') == 0) {
+            toptools.push(m.launcher);
+        }
+    });
+    if (toptools.length > 0){
+        var pnl = Ext.create({
+            xtype: 'desktop-tb',
+            renderTo: 'ux-toptoolbar',
+            tbar: toptools
+        });
+        pnl.doLayout();
+    }
+
+    function minimizeWin(win){
         win.minimized = true;
         win.hide();
-    },
+    }
 
-    markActive: function (win) {
-        if(this.activeWindow && this.activeWindow != win){
-            this.markInactive(this.activeWindow);
+    function markActive(win){
+        if(activeWindow && activeWindow != win){
+            markInactive(activeWindow);
         }
-        this.taskbar.setActiveButton(win.taskButton);
-        this.activeWindow = win;
+        taskbar.setActiveButton(win.taskButton);
+        activeWindow = win;
         Ext.fly(win.taskButton.el).addClass('active-win');
         win.minimized = false;
-    },
+    }
 
-    markInactive: function (win) {
-        if(win == this.activeWindow){
-            this.activeWindow = null;
+    function markInactive(win){
+        if(win == activeWindow){
+            activeWindow = null;
             Ext.fly(win.taskButton.el).removeClass('active-win');
         }
-    },
+    }
 
-    removeWin: function (win){
-        this.taskbar.removeTaskButton(win.taskButton);
-        this.layout();
-    },
+    function removeWin(win){
+        taskbar.removeTaskButton(win.taskButton);
+        layout();
+    }
 
-    layout: function () {
-        var taskbarEl = Ext.get('ux-taskbar'),
-            desktopEl = Ext.get('x-desktop'),
-            toptoolbarEl = Ext.get('ux-toptoolbar'),
-            viewHeight = Ext.lib.Dom.getViewHeight(),
+    function layout(){
+        var viewHeight = Ext.lib.Dom.getViewHeight(),
             taskbarHeight = taskbarEl.getHeight(),
             toptoolbarHeight = toptoolbarEl.getHeight();
 
         desktopEl.setHeight(viewHeight - taskbarHeight - toptoolbarHeight);
-    },
+    }
+    Ext.EventManager.onWindowResize(layout);
+
+    this.layout = layout;
+
+    this.createWindow = function(win, cls){
+        /* D prefer 24.03.10 
+         * modify @config parameter to @win parameter
+         * win - Готовое окно
+         * >>
+        var win = new (cls||Ext.Window)(
+            Ext.applyIf(config||{}, {
+                renderTo: desktopEl,
+                manager: windows,
+                minimizable: true,
+                maximizable: true
+            })
+        );*/
+        //win.render(desktopEl);
+
+        win.taskButton = taskbar.addTaskButton(win);
+
+        //win.cmenu = new Ext.menu.Menu({
+        //    items: []
+        //});
 
-    createWindow: function (win) {
-        win.taskButton = this.taskbar.addTaskButton(win);
         win.animateTarget = win.taskButton.el;
 
-        win.on('activate', this.markActive, this);
-        win.on('beforeshow', this.markActive, this);
-        win.on('deactivate', this.markInactive, this);
-        win.on('minimize', this.minimizeWin, this);
-        win.on('close', this.removeWin, this);
+        win.on({
+            'activate': {
+                fn: markActive
+            },
+            'beforeshow': {
+                fn: markActive
+            },
+            'deactivate': {
+                fn: markInactive
+            },
+            'minimize': {
+                fn: minimizeWin
+            },
+            'close': {
+                fn: removeWin
+            }
+        });
 
-        this.markActive(win);
-        this.layout();
+        layout();
         return win;
-    },
+    };
 
-    getManager: function () {
-        return this.windowManager;
-    },
+    this.getManager = function(){
+        return windows;
+    };
 
-    getWindow: function (id) {
-        return this.getManager().get(id);
-    },
+    this.getWindow = function(id){
+        return windows.get(id);
+    }
 
-    getWinWidth: function () {
+    this.getWinWidth = function(){
         var width = Ext.lib.Dom.getViewWidth();
         return width < 200 ? 200 : width;
-    },
+    }
 
-    getWinHeight: function () {
-        var taskbarEl = Ext.get('ux-taskbar'),
-            height = (Ext.lib.Dom.getViewHeight() - taskbarEl.getHeight());
+    this.getWinHeight = function(){
+        var height = (Ext.lib.Dom.getViewHeight()-taskbarEl.getHeight());
         return height < 100 ? 100 : height;
-    },
+    }
 
-    getWinX: function (width) {
+    this.getWinX = function(width){
         return (Ext.lib.Dom.getViewWidth() - width) / 2
-    },
+    }
 
-    getWinY: function (height) {
-        var taskbarEl = Ext.get('ux-taskbar'),
-            toptoolbarEl = Ext.get('ux-toptoolbar');
+    this.getWinY = function(height){
         return (Ext.lib.Dom.getViewHeight()-taskbarEl.getHeight() - height - toptoolbarEl.getHeight()) / 2;
-    },
+    }
 
-    prepareDesktopItem: function (el) {
-        var shortcuts = Ext.get('x-shortcuts');
-        if (this.enableDragAndDrop) {
-            // добавим возможность тащить
-            var dd = new Ext.dd.DragSource(el, {
-                ddGroup: 'desktopitems',
-                scroll: false,
-                dragData: getDesktopComponent(el)
-            });
-        }
-        el.on(this.primaryEvent, function(e, t){
+    layout();
+
+    // Если в прикладном приложении при создании Ext.app.App передать в словаре
+    // параметр primaryEvent, то срабатывание именно этого эвента запустит
+    // действие, назначенное на ярлык на рабочем столе.
+    // Пример: если передать primaryEvent: 'dblclick', то ярлыки будут
+    // открываться только по даблклику.
+    var primaryEvent = app.primaryEvent || 'click';
+    if(shortcuts){
+        shortcuts.on(primaryEvent, function(e, t){
             var t = e.getTarget('td', shortcuts);
 			if(t){
                 e.stopEvent();
-                getDesktopComponent(t).handler();
+                var module = app.getModule(t.id.replace('-shortcut', ''));
+                if(module){
+                    module.launcher.handler();
+                }
             }
         });
     }
-});
 
-/*!
- * Ext JS Library 3.3.0
- * Copyright(c) 2006-2010 Ext JS, Inc.
- * licensing@extjs.com
- * http://www.extjs.com/license
- */
-Ext.app.Module = Ext.extend(Ext.util.Observable, {
-    constructor: function (config) {
-        Ext.apply(this, config);
-        Ext.app.Module.superclass.constructor.call(this);
+    if (Ext.isIE7) {
+        Ext.EventManager.fireResize();
     }
-});
+};
 
 /*!
  * Ext JS Library 3.3.0
@@ -1257,32 +1025,23 @@ Ext.app.Module = Ext.extend(Ext.util.Observable, {
  * licensing@extjs.com
  * http://www.extjs.com/license
  */
-Ext.app.App = Ext.extend(Ext.util.Observable, {
+Ext.app.App = function(cfg){
+    Ext.apply(this, cfg);
+    this.addEvents({
+        'ready' : true,
+        'beforeunload' : true
+    });
+
+    Ext.onReady(this.initApp, this);
+};
+
+Ext.extend(Ext.app.App, Ext.util.Observable, {
     isReady: false,
     startMenu: null,
     modules: null,
 
-    constructor: function (desktopConfig, params) {
-        this.startMenuIcon = params.startMenuIcon;
-        this.startMenuText = params.startMenuText;
-
-        this.enableDragAndDrop = params.enableDragAndDrop;
-        this.addDesktopItemUrl = params.addDesktopItemUrl;
-        this.removeDesktopItemUrl = params.removeDesktopItemUrl;
-        this.useMenuSearch = params.useMenuSearch;
-
-        this.menuItems = desktopConfig.menuItems;
-        this.desktopItems = desktopConfig.desktopItems;
-        this.topToolbarItems = desktopConfig.topToolbarItems;
-        this.toolboxItems = desktopConfig.toolboxItems;
-
-        this.extraParams = params.extra;
-
-        this.addEvents({
-            'ready': true,
-            'beforeunload': true
-        });
-        Ext.onReady(this.initApp, this);
+    getStartConfig : function(){
+		//
     },
 
     initApp : function(){
@@ -1294,116 +1053,42 @@ Ext.app.App = Ext.extend(Ext.util.Observable, {
 
         this.modules = this.getModules();
         if(this.modules){
-            this.getDesktop().addModules(this.modules);
             this.initModules(this.modules);
         }
 
+        this.init();
+
         Ext.EventManager.on(window, 'beforeunload', this.onUnload, this);
         this.fireEvent('ready', this);
         this.isReady = true;
-
-        if (this.enableDragAndDrop) {
-            var dropzone = new Ext.dd.DropTarget(Ext.get('ux-toptoolbar'), {
-                app: this,
-                ddGroup: 'desktopitems',
-                notifyDrop: function (source, e, data) {
-                    this.app.removeDesktopItem(source.id);
-                    return true;
-                }
-            });
-
-            var dropzone = new Ext.dd.DropTarget(Ext.get('x-desktop'), {
-                app: this,
-                ddGroup: 'toolbaritems',
-                notifyDrop: function (source, e, data) {
-                    this.app.addDesktopItem(source, e, data);
-                    return true;
-                }
-            });
-        }
     },
 
-    addDesktopItem: function(source, e, data){
-        // добавим ярлык на рабочий стол
-        var shortcuts = Ext.query("#x-shortcuts > tbody > tr")[0];
-        var count = shortcuts.querySelectorAll('td').length;
-        var name = data.id + '-shortcut' + count;
-        shortcuts.insertAdjacentHTML(
-            'beforeEnd', this.getShortcutHTML(name, this.getShortcutIconCls(data), data.text));
-        // навесим обработчик
-        this.getDesktop().prepareDesktopItem(Ext.get(name));
-
-        if (this.addDesktopItemUrl){
-            Ext.Ajax.request({
-                url: this.addDesktopItemUrl,
-                params: this.getAddDesktopItemParams(data),
-                method: 'POST',
-                failure: function(){
-                    uiAjaxFailMessage.apply(this, arguments);
-                },
-                scope: this
-            });
-        }
-    },
-
-    getShortcutIconCls: function (data) {
-        return 'default-launcher';
-    },
-
-    getShortcutHTML: function (id, iconCls, text) {
-        return String.format(
-            '<td id="{0}"><a href="#"><div class="base-desktop-image {1}"></div><div class="desktop-item-shortcut-label">{2}</div></a></td>',
-            id, iconCls, text
-        );
-    },
-
-    getAddDesktopItemParams: function (data) {
-        return {}
-    },
-
-    getRemoveDesktopItemParams: function (data) {
-        return {}
-    },
-
-    removeDesktopItem: function(id){
-        var el = Ext.get(id),
-            cmp = getDesktopComponent(el);
-        el.removeAllListeners();
-        // уберем ярлык с рабочего стола
-        Ext.query('#' + el.id)[0].remove();
-
-        if (this.removeDesktopItemUrl){
-            Ext.Ajax.request({
-                url: this.removeDesktopItemUrl,
-                params: this.getRemoveDesktopItemParams(cmp),
-                method: 'POST',
-                failure: function(){
-                    uiAjaxFailMessage.apply(this, arguments);
-                },
-                scope: this
-            });
-        }
-    },
+    getModules : Ext.emptyFn,
+    init : Ext.emptyFn,
 
     initModules : function(ms){
         for(var i = 0, len = ms.length; i < len; i++){
             var m = ms[i];
+			// M prefer 23.03.10 >>
+            // this.launcher.add(m.launcher);	
+            // -->
             if (m && m.launcher && m.launcher.in_start_menu == true) {
-                this.launcher.add(m.launcher);
-            }
-           if (m) {
+            	this.launcher.add(m.launcher);		
+            };
+            // prefer <<
+           if (m) { 
                 m.app = this;
            }
         }
     },
 
     getModule : function(name){
-        var ms = this.modules;
-        for(var i = 0, len = ms.length; i < len; i++){
+    	var ms = this.modules;
+    	for(var i = 0, len = ms.length; i < len; i++){
             //ZIgi 23.02.11
-            if(ms[i] && ( ms[i].id == name || ms[i].appType == name)){
-                return ms[i];
-            }
+    		if(ms[i] && ( ms[i].id == name || ms[i].appType == name)){
+    			return ms[i];
+			}
         }
         return '';
     },
@@ -1424,109 +1109,64 @@ Ext.app.App = Ext.extend(Ext.util.Observable, {
         if(this.fireEvent('beforeunload', this) === false){
             e.stopEvent();
         }
-    },
-
-    // Реализация функции, которая выводит список
-    getModules : function(){
-        var res = [];
-
-        // пункты Главного Меню
-        this.addEachModuleTo(
-            res, this.menuItems, 'menu-item-', true
-        );
-
-        // элементы собственно Рабочего Стола
-        this.addEachModuleTo(
-            res, this.desktopItems, 'desktop-item-'
-        );
-
-        // верхняя панель
-        this.addEachModuleTo(
-            res, this.topToolbarItems, 'toptoolbar-item-'
-        );
-
-        res = this.postprocessModules(res);
-        return res;
-    },
-
-    postprocessModules: function(modules) {
-        return modules;
-    },
-
-    // Обязательные настройки меню "Пуск"
-    getStartConfig: function() {
-        var items = [];
-        for (var i = 0; i < this.toolboxItems.length; i += 1) {
-            var item = this.toolboxItems[i];
-            items.push(this.makeItem(item));
-        }
-        return {
-            toolItems: items,
-            title: this.startMenuText,
-            toolsPanelWidth: 120,
-            width: 330,
-            iconCls: this.startMenuIcon
-        };
-    },
-
-    /**
-    * Преобразователь конфигов для элементов в собственно элементы
-    */
-    makeItem: function (data) {
-        var res;
-        if (data == '-') {
-            res = data; // разделитель
-        } else {
-            res = {
-                scope: this,
-                text: data.text,
-                iconCls: data.icon
-            };
-            if (data.items != undefined && data.items.length > 0) {
-                res.handler = function() { return false; };
-                res.menu = [];
-                for (var i = 0; i < data.items.length; i += 1) {
-                    res.menu.push(this.makeItem(data.items[i]))
-                }
-            } else {
-                res.handler = function() {
-                    sendRequest(
-                        data.url,
-                        this.getDesktop(),
-                        data.context
-                    );
-                }.bind(this);
-            }
-        }
-        Ext.applyIf(res, data.extra || {});
-        return res;
-    },
-
-    /**
-     * Генератор модулей - потомков от Ext.app.Module
-     */
-    makeModule: function (data, idPrefix, idx, inStartMenu) {
-        /**
-         * Генератор словарей, описывающих элементы (под)меню
-         */
-        assert(data != '-', "Separator at top level!");
-        var module = new Ext.app.Module();
-        module.id = idPrefix + idx;
-        module.launcher = this.makeItem(data);
-        if (inStartMenu) {
-            module.launcher.in_start_menu = true;
-        }
-        return module;
-    },
-
-    /**
-     * Добавляет каждый элемент @source@ в @target@, как модуль
-     * с id вида (@prefix@ + N).
-     * Флаг inStartMenu=true добавит модуль в меню
-     */
-    addEachModuleTo: function (target, source, prefix, inStartMenu) {
-        for(var i = 0; i < source.length; i += 1) {
-            target.push(this.makeModule(source[i], prefix, i + 1, inStartMenu));
-        }
     }
 });
+
+/*!
+ * Ext JS Library 3.3.0
+ * Copyright(c) 2006-2010 Ext JS, Inc.
+ * licensing@extjs.com
+ * http://www.extjs.com/license
+ */
+Ext.app.Module = function(config){
+    Ext.apply(this, config);
+    Ext.app.Module.superclass.constructor.call(this);
+    this.init();
+}
+
+Ext.extend(Ext.app.Module, Ext.util.Observable, {
+    init : Ext.emptyFn
+});
+
+Ext.define('Ext.m3.DesktopTopToolbar', {
+    extend: 'Ext.Panel',
+    xtype:'desktop-tb',
+    monitorResize: true,
+    autoWidth: true,
+    autoHeight: true,
+    height: 0,
+    style: 'margin-top:0px',
+    bodyStyle: 'padding:0px',
+    autoScroll: true,
+    tbar: [],
+    initComponent : function() {
+        this.tbar = this.replaceToolbar(this.tbar);
+        this.callParent(this);
+        Ext.EventManager.onWindowResize(this.fireResize, this);
+    },
+    fireResize : function(w, h){
+        this.onResize(w, 0, w, 0);
+        this.fireEvent('resize', this, w, h, w, h);
+    },
+    replaceToolbar: function(items){
+        var newItems = items;
+        if (items && Ext.isArray(items)) {
+            newItems = [];
+            Ext.each(items, function(item){
+                newItems.push(this.replaceTBItem(item));
+            }, this);
+        }
+        return newItems;
+    },
+    replaceTBItem: function(item){
+        if (item.text == 'FILLBLOCK') {
+            return '->';
+        } else if(item.text == 'TIMEBLOCK'){
+            return new Ext.ux.Clock();
+        } else if(item.text == '-'){
+            return '-';
+        } else {
+            return item;
+        }
+    }
+});
\ No newline at end of file
