@@ -4,84 +4,57 @@
  * @param {Object} baseConfig
  * @param {Object} params
  */
+Ext.define('Ext.m3.AdvancedComboBox', {
+    extend: 'Ext.m3.ComboBox',
+    xtype: 'm3-select',
 
-Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
-    constructor: function (baseConfig, params) {
+    bubbleEvents: [
+        'mask',
+        'unmask',
+        'getcontext'
+    ],
 
-        /**
-         * Инициализация значений
-         */
+    askBeforeDeleting: true,
 
-            // Будет ли задаваться вопрос перед очисткой значения
-        this.askBeforeDeleting = true;
+    actionSelectUrl: null,
+    actionEditUrl: null,
+    autocompleteUrl: null,
 
-        this.actionSelectUrl = null;
-        this.actionEditUrl = null;
-        this.actionContextJson = null;
+    hideBaseTrigger: false,
 
-        this.hideBaseTrigger = false;
+    defaultValue: null,
+    defaultText: null,
+    defaultRecord: null,
 
-        this.defaultValue = null;
-        this.defaultText = null;
-        this.defaultRecord = null;
+    // кнопка очистки
+    hideTriggerClear: false,
+    hideTriggerDropDown: false,
+    hideTriggerDictSelect: false,
+    hideTriggerDictEdit: true,
 
-        // кнопка очистки
-        this.hideTriggerClear = params.hideClearTrigger || false;
+    defaultLimit: '50',
 
-        // кнопка выбора из выпадающего списка
-        this.hideTriggerDropDown = false;
+    triggerClearClass: 'x-form-clear-trigger',
+    triggerSelectClass: 'x-form-select-trigger',
+    triggerEditClass: 'x-form-edit-trigger',
 
-        // кнопка выбора из справочника
-        this.hideTriggerDictSelect = params.hideDictSelectTrigger || false;
 
-        // кнопка редактирования элемента
-        this.hideTriggerDictEdit = true;
-        if (!params.hideEditTrigger) {
-            this.hideTriggerDictEdit = params.hideEditTrigger;
+    /**
+     * Конфигурация компонента
+     */
+    initComponent: function () {
+        if (!this.store.url && this.autocompleteUrl) {
+            this.store.url = this.autocompleteUrl;
         }
 
-        // Количество записей, которые будут отображаться при нажатии на кнопку
-        // выпадающего списка
-        this.defaultLimit = '50';
-
-        // css классы для иконок на триггеры
-        this.triggerClearClass = 'x-form-clear-trigger';
-        this.triggerSelectClass = 'x-form-select-trigger';
-        this.triggerEditClass = 'x-form-edit-trigger';
-
-
-        assert(params.actions, 'params.actions is undefined');
-
-        if (params.actions.actionSelectUrl) {
-            this.actionSelectUrl = params.actions.actionSelectUrl;
-        }
-
-        if (params.actions.actionEditUrl) {
-            this.actionEditUrl = params.actions.actionEditUrl;
-        }
-
-        this.askBeforeDeleting = params.askBeforeDeleting;
-        this.actionContextJson = params.actions.contextJson;
-
-        this.hideBaseTrigger = false;
-        if (baseConfig.hideTrigger) {
-            delete baseConfig.hideTrigger;
-            this.hideBaseTrigger = true;
-        }
-
-
-        this.defaultValue = params.defaultValue;
-        this.defaultText = params.defaultText;
-        this.defaultRecord = Ext.decode(params.recordValue);
-
-        this.baseTriggers = [
+        this.allTriggers = [
             {
                 iconCls: 'x-form-clear-trigger',
                 handler: null,
                 hide: null
             },
             {
-                iconCls: 'x-form-dropdown-trigger',
+                iconCls: '',
                 handler: null,
                 hide: null
             },
@@ -96,29 +69,22 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
                 hide: true
             }
         ];
-        this.allTriggers = [].concat(this.baseTriggers);
-        if (params.customTriggers) {
-            Ext.each(params.customTriggers, function (item, index, all) {
-                this.allTriggers.push(item);
-            }, this);
 
-        }
-
-        Ext.m3.AdvancedComboBox.superclass.constructor.call(this, baseConfig);
-    },
-    /**
-     * Конфигурация компонента
-     */
-    initComponent: function () {
         Ext.m3.AdvancedComboBox.superclass.initComponent.call(this);
 
+        if (this.hideTrigger) {
+            delete this.hideTrigger;
+            this.hideBaseTrigger = true;
+        }
+
         // см. TwinTriggerField
         this.triggerConfig = {
             tag: 'span', cls: 'x-form-twin-triggers', cn: []};
 
         Ext.each(this.allTriggers, function (item, index, all) {
             this.triggerConfig.cn.push(
-                {tag: "img", src: Ext.BLANK_IMAGE_URL, cls: "x-form-trigger " + item.iconCls}
+                {tag: "img", src: Ext.BLANK_IMAGE_URL,
+                    cls: "x-form-trigger " + item.iconCls}
             );
         }, this);
 
@@ -135,7 +101,14 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
         }
 
         // Значения по-умолчанию
-        this.initDefault();
+        if (this.defaultRecord && Object.getOwnPropertyNames(this.defaultRecord).length !== 0) { // Проверка на пустоту объекта
+            var record = new Ext.data.Record(this.defaultRecord);
+            this.setRecord(record);
+        } else {
+            if (this.defaultValue && this.defaultText) {
+                this.addRecordToStore(this.defaultValue, this.defaultText);
+            }
+        }
 
         this.validator = this.validateField;
 
@@ -160,7 +133,6 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
              *   this - Сам компонент
              *   id - Значение
              *   text - Текстовое представление значения
-             *   extraData - Словарь с дополнительными данными из выборанного значения
              * Возвр. значения:
              *   true - обработка продолжается
              *   false - отмена обработки
@@ -178,21 +150,11 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
             'changed'
         );
 
-        this.getStore().baseParams = Ext.applyIf({start: 0, limit: this.defaultLimit }, this.getStore().baseParams);
+        this.getStore().baseParams = Ext.applyIf({start: 0,
+                limit: this.defaultLimit },
+            this.getStore().baseParams);
         this.triggerAction = 'all';
-    },
-    /**
-     * Установка значений по-умолчанию
-     */
-    initDefault: function () {
-        if (this.defaultRecord) {
-            var record = new Ext.data.Record(this.defaultRecord);
-            this.setRecord(record);
-        } else {
-            if (this.defaultValue && this.defaultText) {
-                this.addRecordToStore(this.defaultValue, this.defaultText);
-            }
-        }
+
     },
     // см. TwinTriggerField
     getTrigger: function (index) {
@@ -207,25 +169,15 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
             var triggerIndex = 'Trigger' + (index + 1);
             t.hide = function () {
                 var w = triggerField.wrap.getWidth();
-                if (w === 0) {
-                    w = triggerField.bufferSize;
-                    triggerField.wrap.setWidth(w);
-                }
-
                 this.dom.style.display = 'none';
-                triggerField['hidden' + triggerIndex] = true;
-                triggerField.el.setWidth(w - triggerField.getTriggerWidth());
+                triggerField.el.setWidth(w - triggerField.trigger.getWidth());
+                this['hidden' + triggerIndex] = true;
             };
             t.show = function () {
                 var w = triggerField.wrap.getWidth();
-                if (w === 0) {
-                    w = triggerField.bufferSize;
-                    triggerField.wrap.setWidth(w);
-                }
-
                 this.dom.style.display = '';
-                triggerField['hidden' + triggerIndex] = false;
-                triggerField.el.setWidth(w - triggerField.getTriggerWidth());
+                triggerField.el.setWidth(w - triggerField.trigger.getWidth());
+                this['hidden' + triggerIndex] = false;
             };
 
             if (this.allTriggers[index].hide) {
@@ -266,29 +218,30 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
                 }
             }, this);
         } else {
-            this.baseTriggers[0].hide = disabled;
-            this.baseTriggers[1].hide = disabled;
-            this.baseTriggers[2].hide = disabled;
-            this.baseTriggers[3].hide = disabled;
+            this.allTriggers[0].hide = disabled;
+            this.allTriggers[1].hide = disabled;
+            this.allTriggers[2].hide = disabled;
+            this.allTriggers[3].hide = disabled;
         }
-    }
+    },
 
     /**
      * Инициализация первоначальной настройки триггеров
-     */, initBaseTrigger: function () {
-        this.baseTriggers[0].handler = this.onTriggerClearClick;
-        this.baseTriggers[1].handler = this.onTriggerDropDownClick;
-        this.baseTriggers[2].handler = this.onTriggerDictSelectClick;
-        this.baseTriggers[3].handler = this.onTriggerDictEditClick;
+     */
+    initBaseTrigger: function () {
+        this.allTriggers[0].handler = this.onTriggerClearClick;
+        this.allTriggers[1].handler = this.onTriggerDropDownClick;
+        this.allTriggers[2].handler = this.onTriggerDictSelectClick;
+        this.allTriggers[3].handler = this.onTriggerDictEditClick;
 
-        this.baseTriggers[0].hide = this.hideTriggerClear || this.readOnly || this.disabled;
-        this.baseTriggers[1].hide = this.hideTriggerDropDown || this.readOnly || this.disabled;
-        this.baseTriggers[2].hide = this.hideTriggerDictSelect || this.readOnly || this.disabled;
-        this.baseTriggers[3].hide = this.hideTriggerDictEdit || this.readOnly || this.disabled;
+        this.allTriggers[0].hide = this.hideTriggerClear || this.readOnly || this.disabled;
+        this.allTriggers[1].hide = this.hideTriggerDropDown || this.readOnly || this.disabled;
+        this.allTriggers[2].hide = this.hideTriggerDictSelect || this.readOnly || this.disabled;
+        this.allTriggers[3].hide = this.hideTriggerDictEdit || this.readOnly || this.disabled;
 
         if (!this.getValue()) {
-            this.baseTriggers[0].hide = true;
-            this.baseTriggers[3].hide = true;
+            this.allTriggers[0].hide = true;
+            this.allTriggers[3].hide = true;
         }
     },
 
@@ -312,7 +265,6 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
     onDestroy: function () {
         Ext.destroy(this.triggers);
         Ext.destroy(this.allTriggers);
-        Ext.destroy(this.baseTriggers);
         Ext.m3.AdvancedComboBox.superclass.onDestroy.call(this);
     },
 
@@ -326,11 +278,17 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
                 this.collapse();
             } else {
                 this.onFocus({});
-                this.doQuery(this.allQuery, true);
+                this.doQuery(null, true);
             }
             this.el.focus();
         }
     },
+
+    getParams: function (q) {
+        var result = Ext.m3.AdvancedComboBox.superclass.getParams.call(this, q);
+        return Ext.applyIf(result, this.getContext());
+    },
+
     /**
      * Кнопка открытия справочника в режиме выбора
      */
@@ -370,12 +328,11 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
      * @param {Object} index
      */
     onSelect: function (record, index) {
-        var oldValue = this.getValue();
         if (this.fireEvent('afterselect', this, record.data[this.valueField], record.data[this.displayField])) {
             Ext.m3.AdvancedComboBox.superclass.onSelect.call(this, record, index);
             this.showClearBtn();
             this.showEditBtn();
-            this.fireEvent('change', this, record.data[this.valueField || this.displayField], oldValue);
+            this.fireEvent('change', this, record.data[this.valueField || this.displayField]);
             this.fireEvent('changed', this);
         }
     },
@@ -399,46 +356,6 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
             this.hiddenTrigger1 = true;
         }
     },
-    /**
-     * Показывает кнопку выбора из справочника
-     */
-    showDictSelectBtn: function () {
-        if (!this.hideTriggerDictSelect && this.rendered && !this.readOnly && !this.disabled) {
-            this.getTrigger(2).show();
-        } else {
-            this.hiddenTrigger3 = false || this.hideTriggerDictSelect || this.readOnly || this.disabled;
-        }
-    },
-    /**
-     * Скрывает кнопку выбора из справочника
-     */
-    hideDictSelectBtn: function () {
-        if (this.rendered) {
-            this.getTrigger(2).hide();
-        } else {
-            this.hiddenTrigger3 = true;
-        }
-    },
-    /**
-     * Показывает кнопку выпадающего списка
-     */
-    showDropDownBtn: function () {
-        if (!this.hideTriggerDropDown && this.rendered && !this.readOnly && !this.disabled) {
-            this.getTrigger(1).show();
-        } else {
-            this.hiddenTrigger2 = false || this.hideTriggerDropDown || this.readOnly || this.disabled;
-        }
-    },
-    /**
-     * Скрывает кнопку выпадающего списка
-     */
-    hideDropDownBtn: function () {
-        if (this.rendered) {
-            this.getTrigger(1).hide();
-        } else {
-            this.hiddenTrigger2 = true;
-        }
-    },
     /**
      * Показывает кнопку открытия карточки элемента
      */
@@ -477,6 +394,11 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
      * кнопку очистки
      */
     setValue: function (value) {
+        if (Ext.isObject(value)) {
+            // binding - добавим запись в store
+            this.getStore().add(new Ext.data.Record(value));
+            value = value['id'];
+        }
         Ext.m3.AdvancedComboBox.superclass.setValue.call(this, value);
         if (value) {
             this.showClearBtn();
@@ -485,55 +407,29 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
     },
     /**
      * Генерирует ajax-запрос за формой выбора из справочника и
-     * вешает обработку на предопределенное событие closed_ok
+     * вешает обработку на предопределенное событие select
      */
     onSelectInDictionary: function () {
         assert(this.actionSelectUrl, 'actionSelectUrl is undefined');
 
-        if (this.fireEvent('beforerequest', this)) {
-
-            var parentWin = Ext.getCmp(this.actionContextJson['m3_window_id']),
-                mask;
-            if (parentWin) {
-                mask = new Ext.LoadMask(parentWin.getEl(),
-                    {msg: "Пожалуйста выберите элемент...", msgCls: 'x-mask'});
-                mask.show();
-            }
-
-            Ext.Ajax.request({
-                url: this.actionSelectUrl,
-                method: 'POST',
-                params: this.actionContextJson,
-                success: function (response, opts) {
-                    try {
-                        var win = smart_eval(response.responseText);
-                        if (win) {
-
-                            win.on('closed_ok', function (id, displayText, additionalInfo) {
-                                if (this.fireEvent('afterselect', this, id, displayText, additionalInfo)) {
-                                    this.addRecordToStore(id, displayText, additionalInfo);
-                                }
-                            }, this);
-                            if (mask) {
-                                win.on('close', function () {
-                                    mask.hide();
-                                });
-                            }
-                        }
-                    } finally {
-                        if (mask) {
-                            mask.hide();
-                        }
+        var req = {
+            mode: "Пожалуйста выберите элемент...",
+            url: this.actionSelectUrl,
+            params: Ext.apply({}, this.getContext()),
+            failure: uiAjaxFailMessage,
+            success: function (win) {
+                win.on('select', function (cmp, id, displayText) {
+                    if (this.fireEvent('afterselect', this, id, displayText)) {
+                        this.addRecordToStore(id, displayText);
                     }
-                },
-                failure: function (response, opts) {
-                    if (mask) {
-                        mask.hide();
-                    }
-                    uiAjaxFailMessage.apply(this, arguments);
-                },
-                scope: this
-            });
+                }, this);
+                return win;
+            }.bind(this)
+        };
+
+        if (this.fireEvent('beforerequest', this, req)) {
+
+            UI.callAction.call(this, req);
         }
     },
     /**
@@ -609,7 +505,7 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
         Ext.Ajax.request({
             url: this.actionEditUrl,
             method: 'POST',
-            params: Ext.applyIf({id: value_id}, this.actionContextJson),
+            params: Ext.applyIf({id: value_id}, this.getContext()),
             success: function (response, opts) {
                 smart_eval(response.responseText);
             },
@@ -648,26 +544,22 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
      * Поведение зависит от выбранного флага show
      */
     showTriggers: function (show) {
-
         if (show) {
             if (this.getValue()) {
                 this.showClearBtn();
                 this.showEditBtn();
             }
-            this.showDictSelectBtn();
-            this.showDropDownBtn();
-        } else {
-            if (this.triggers){ // this.triggers появляется только после рендера
-                this.hideClearBtn();
-                this.hideEditBtn();
-                this.hideDictSelectBtn();
-                this.hideDropDownBtn();
-            } else { // иначе достаточно просто пометить соответствующие поля
-                this.hiddenTrigger1 = true; // для правильного расчета
-                this.hiddenTrigger2 = true; // ширины триггеров
-                this.hiddenTrigger3 = true;
-                this.hiddenTrigger4 = true;
+            if (!this.hideTriggerDictSelect) {
+                this.getTrigger(2).show();
             }
+            if (!this.hideTriggerDropDown) {
+                this.getTrigger(1).show();
+            }
+        } else {
+            this.hideClearBtn();
+            this.hideEditBtn();
+            this.getTrigger(2).hide();
+            this.getTrigger(1).hide();
         }
     },
 
@@ -688,22 +580,16 @@ Ext.m3.AdvancedComboBox = Ext.extend(Ext.m3.ComboBox, {
      */
     setReadOnly: function (readOnly) {
         var width = this.getWidth();
-        if (width === 0) {
-            width = this.bufferSize;
-        }
-
-        Ext.m3.AdvancedComboBox.superclass.setReadOnly.call(this, readOnly);
-
         this.disableTriggers(readOnly);
-        this.showTriggers(!readOnly); // скрытие/показ триггеров
-
+        Ext.m3.AdvancedComboBox.superclass.setReadOnly.call(this, readOnly);
         if (readOnly) {
             this.el.setWidth(width);
             if (this.wrap) this.wrap.setWidth(width);
         } else {
+
+            this.showTriggers(!readOnly);
+
             this.onResize(width);
         }
     }
 });
-
-Ext.reg('m3-select', Ext.m3.AdvancedComboBox);
