@@ -1,32 +1,26 @@
-# coding: utf-8
-u"""Классы для работы первично отображаемого интерфейса MIS.
-
+#coding:utf-8
+"""
+Классы для работы первично отображаемого интерфейса MIS.
 Включают список модулей в меню "Пуск" и список модулей на "Рабочем столе"
 """
-from __future__ import absolute_import
 
-from importlib import import_module
-from logging import getLogger
-from uuid import uuid4
-import itertools
 import threading
+import inspect
+import itertools
 import warnings
+from uuid import uuid4
 
 from django.conf import settings
-from m3 import M3JSONEncoder
-from m3.actions import Action
-from m3.actions import ControllerCache
-from m3_django_compat import get_installed_apps
-import six
-from six.moves import map
-from six.moves import filter
-
-
-logger = getLogger('django')
-
+from django.utils.importlib import import_module
+from django.contrib.auth.models import User, AnonymousUser
+try:
+    from django.utils.log import logger
+except ImportError:
+    from django.utils.log import getLogger
+    logger = getLogger('django')
 
 try:
-    from m3_users.app_meta import GENERIC_USER, SUPER_ADMIN
+    from m3_users import GENERIC_USER, SUPER_ADMIN
     from m3_users.helpers import get_assigned_metaroles_query
     from m3_users.metaroles import UserMetarole, get_metarole
 except ImportError:
@@ -49,6 +43,9 @@ except ImportError:
     def get_metarole(code):
         return UserMetarole()
 
+from m3.actions import ControllerCache, Action, ActionPack
+from m3 import M3JSONEncoder
+
 
 # Константы: "Разделитель", "Блок с текущим временем", "Заполняющий блок"
 SEPARATOR = '-'
@@ -91,17 +88,17 @@ class DesktopElementCollection(object):
                 # на всякий случай клонируем
                 item = item.clone()
                 item.subitems = list(sorted(
-                    filter(
+                    itertools.ifilter(
                         bool,
-                        map(
+                        itertools.imap(
                             prepared,
                             item.subitems)),
                     key=self.sorting_key
                 ))
-                result = any(
-                    not (isinstance(i, MenuSeparator) or i.name in ('-', u'-'))
-                    for i in item.subitems
-                ) and item or None
+                if not item.subitems:
+                    result = None
+                else:
+                    result = item
             return result
 
         # обертка списка элементов в некое подобие LaunchGroup
@@ -127,15 +124,26 @@ class DesktopModel(object):
     (start_menu и toolbox) в меню пуск
     и список модулей на Рабочем Столе (desktop)
     """
-    def __init__(self, request, filter_factory):
-        self.start_menu = DesktopElementCollection(
-            filter_factory(request, DesktopLoader.START_MENU))
-        self.toolbox = DesktopElementCollection(
-            filter_factory(request, DesktopLoader.TOOLBOX))
-        self.desktop = DesktopElementCollection(
-            filter_factory(request, DesktopLoader.DESKTOP))
-        self.toptoolbar = DesktopElementCollection(
-            filter_factory(request, DesktopLoader.TOPTOOLBAR))
+    def __init__(self, request):
+        def filter_by_permissions(elem):
+            """
+            Возвращает True, если у ползователя есть права на пак элемента.
+            Работет только с DesktopShortcut'ами - у них есть атрибут pack,
+            остальные же элементы - отображаются всегда
+            :param elem: элемент рабочего стола
+            :type elem: DesktopShortcut
+            :return: has_perm - наличие права доступа к элементу
+            :rtype: bool
+            """
+            pack = getattr(elem, 'pack', None)
+            if pack is None or isinstance(pack, ActionPack):
+                return True
+            else:
+                return pack.has_perm(request)
+        self.start_menu = DesktopElementCollection(filter_by_permissions)
+        self.toolbox = DesktopElementCollection(filter_by_permissions)
+        self.desktop = DesktopElementCollection(filter_by_permissions)
+        self.toptoolbar = DesktopElementCollection(filter_by_permissions)
 
 
 class BaseDesktopElement(object):
@@ -180,18 +188,9 @@ class BaseDesktopElement(object):
     def __eq__(self, other):
         return isinstance(other, BaseDesktopElement) and self.id == other.id
 
-    def __hash__(self):
-        return hash(self.id)
-
-    def __lt__(self, other):
-        assert isinstance(other, BaseDesktopElement), type(other)
-
-        if self.index == other.index:
-            result = self.name < other.name
-        else:
-            result = self.index < other.index
-
-        return result
+    def __cmp__(self, other):
+        assert isinstance(other, BaseDesktopElement)
+        return cmp(self.index, other.index) or cmp(self.name, other.name)
 
     def clone(self, *args, **kwargs):
         """
@@ -286,7 +285,7 @@ class DesktopShortcut(DesktopLauncher):
             if not getattr(pack, 'title', None):
                 pack.title = getattr(pack.parent, 'title', '???')
         else:
-            if isinstance(pack, six.string_types):
+            if isinstance(pack, basestring):
                 # Пробуем найти как пак
                 pack = ControllerCache.find_pack(pack)
                 if not pack:
@@ -349,10 +348,10 @@ class DesktopLoader(object):
                 cls._cache = {}
                 # Из инитов всех приложения
                 # пытаемся выполнить register_desktop_menu
-                for app_name in get_installed_apps():
+                for app_name in settings.INSTALLED_APPS:
                     try:
                         module = import_module('.app_meta', app_name)
-                    except ImportError as err:
+                    except ImportError, err:
                         if err.args[0].find('No module named') == -1 or (
                                 err.args[0].find('app_meta') == -1):
                             logger.exception(
@@ -443,12 +442,12 @@ class DesktopLoader(object):
         :param sorting: функция сортировки
         :type sorting: callable
         """
-        from django.contrib.auth.models import AnonymousUser
+        assert isinstance(user, (User, AnonymousUser))
 
         roles = []
         if not isinstance(user, AnonymousUser):
             roles.extend(get_assigned_metaroles_query(user))
-            if getattr(user, 'is_superuser', False):
+            if user.is_superuser:
                 roles.append(SUPER_ADMIN)
 
         cls.populate_desktop(desktop, roles, sorting)
@@ -550,23 +549,23 @@ class DesktopLoader(object):
                 for role in metarole.get_owner_metaroles():
                     insert_for_role(role, element, processed_metaroles)
 
-        # ===============================================
+        #===============================================
         assert place in (
             cls.DESKTOP,
             cls.START_MENU,
             cls.TOOLBOX,
             cls.TOPTOOLBAR
         )
-        if isinstance(metarole, six.string_types):
+        if isinstance(metarole, basestring):
             metarole = get_metarole(metarole)
         assert isinstance(metarole, UserMetarole)
 
         insert_for_role(metarole, element, processed_metaroles=[])
 
 
-# =============================================================================
+#==============================================================================
 # Разные полезные шорткаты
-# =============================================================================
+#==============================================================================
 
 def add_desktop_launcher(
     name='', url='', icon='', path=None, metaroles=None, places=None
@@ -624,7 +623,7 @@ def add_desktop_launcher(
         launcher = root  # мы в ланчеры, значится, будем добавлять самого рута.
 
     for metarole in cleaned_metaroles:
-        if isinstance(metarole, six.string_types):
+        if isinstance(metarole, basestring):
             metarole = get_metarole(metarole)
         if metarole:
             for place in cleaned_places:
